@@ -92,28 +92,28 @@ __isl_give isl_space *isl_qpolynomial_fold_get_space(
 }
 
 __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_reset_domain_space(
-	__isl_take isl_qpolynomial_fold *fold, __isl_take isl_space *dim)
+	__isl_take isl_qpolynomial_fold *fold, __isl_take isl_space *space)
 {
 	int i;
 
 	fold = isl_qpolynomial_fold_cow(fold);
-	if (!fold || !dim)
+	if (!fold || !space)
 		goto error;
 
 	for (i = 0; i < fold->n; ++i) {
 		fold->qp[i] = isl_qpolynomial_reset_domain_space(fold->qp[i],
-							isl_space_copy(dim));
+							isl_space_copy(space));
 		if (!fold->qp[i])
 			goto error;
 	}
 
 	isl_space_free(fold->dim);
-	fold->dim = dim;
+	fold->dim = space;
 
 	return fold;
 error:
 	isl_qpolynomial_fold_free(fold);
-	isl_space_free(dim);
+	isl_space_free(space);
 	return NULL;
 }
 
@@ -148,16 +148,29 @@ int isl_qpolynomial_fold_involves_dims(__isl_keep isl_qpolynomial_fold *fold,
 	return 0;
 }
 
+/* Given a dimension type for an isl_qpolynomial_fold,
+ * return the corresponding type for the domain.
+ */
+static enum isl_dim_type domain_type(enum isl_dim_type type)
+{
+	if (type == isl_dim_in)
+		return isl_dim_set;
+	return type;
+}
+
 __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_set_dim_name(
 	__isl_take isl_qpolynomial_fold *fold,
 	enum isl_dim_type type, unsigned pos, const char *s)
 {
 	int i;
+	enum isl_dim_type set_type;
 
 	fold = isl_qpolynomial_fold_cow(fold);
 	if (!fold)
 		return NULL;
-	fold->dim = isl_space_set_dim_name(fold->dim, type, pos, s);
+
+	set_type = domain_type(type);
+	fold->dim = isl_space_set_dim_name(fold->dim, set_type, pos, s);
 	if (!fold->dim)
 		goto error;
 
@@ -174,16 +187,6 @@ error:
 	return NULL;
 }
 
-/* Given a dimension type for an isl_qpolynomial_fold,
- * return the corresponding type for the domain.
- */
-static enum isl_dim_type domain_type(enum isl_dim_type type)
-{
-	if (type == isl_dim_in)
-		return isl_dim_set;
-	return type;
-}
-
 __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_drop_dims(
 	__isl_take isl_qpolynomial_fold *fold,
 	enum isl_dim_type type, unsigned first, unsigned n)
@@ -223,6 +226,7 @@ __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_insert_dims(
 	enum isl_dim_type type, unsigned first, unsigned n)
 {
 	int i;
+	enum isl_dim_type set_type;
 
 	if (!fold)
 		return NULL;
@@ -232,7 +236,9 @@ __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_insert_dims(
 	fold = isl_qpolynomial_fold_cow(fold);
 	if (!fold)
 		return NULL;
-	fold->dim = isl_space_insert_dims(fold->dim, type, first, n);
+
+	set_type = domain_type(type);
+	fold->dim = isl_space_insert_dims(fold->dim, set_type, first, n);
 	if (!fold->dim)
 		goto error;
 
@@ -699,12 +705,13 @@ static __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_zero_in_space(
 #undef DEFAULT_IS_ZERO
 #define DEFAULT_IS_ZERO 1
 
-#define NO_NEG
-#define NO_SUB
-#define NO_PULLBACK
-
 #include <isl_pw_templ.c>
 #include <isl_pw_eval.c>
+#include <isl_pw_insert_dims_templ.c>
+#include <isl_pw_lift_templ.c>
+#include <isl_pw_morph_templ.c>
+#include <isl_pw_move_dims_templ.c>
+#include <isl_pw_opt_templ.c>
 
 #undef BASE
 #define BASE pw_qpolynomial_fold
@@ -715,9 +722,9 @@ static __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_zero_in_space(
 #include <isl_union_eval.c>
 
 __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_empty(enum isl_fold type,
-	__isl_take isl_space *dim)
+	__isl_take isl_space *space)
 {
-	return qpolynomial_fold_alloc(type, dim, 0);
+	return qpolynomial_fold_alloc(type, space, 0);
 }
 
 __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_alloc(
@@ -1346,15 +1353,15 @@ enum isl_fold isl_union_pw_qpolynomial_fold_get_type(
 }
 
 __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_lift(
-	__isl_take isl_qpolynomial_fold *fold, __isl_take isl_space *dim)
+	__isl_take isl_qpolynomial_fold *fold, __isl_take isl_space *space)
 {
 	int i;
 
-	if (!fold || !dim)
+	if (!fold || !space)
 		goto error;
 
-	if (isl_space_is_equal(fold->dim, dim)) {
-		isl_space_free(dim);
+	if (isl_space_is_equal(fold->dim, space)) {
+		isl_space_free(space);
 		return fold;
 	}
 
@@ -1363,23 +1370,23 @@ __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_lift(
 		goto error;
 
 	isl_space_free(fold->dim);
-	fold->dim = isl_space_copy(dim);
+	fold->dim = isl_space_copy(space);
 	if (!fold->dim)
 		goto error;
 
 	for (i = 0; i < fold->n; ++i) {
 		fold->qp[i] = isl_qpolynomial_lift(fold->qp[i],
-						isl_space_copy(dim));
+						isl_space_copy(space));
 		if (!fold->qp[i])
 			goto error;
 	}
 
-	isl_space_free(dim);
+	isl_space_free(space);
 
 	return fold;
 error:
 	isl_qpolynomial_fold_free(fold);
-	isl_space_free(dim);
+	isl_space_free(space);
 	return NULL;
 }
 
@@ -1645,7 +1652,7 @@ __isl_give isl_union_pw_qpolynomial_fold *isl_union_map_apply_union_pw_qpolynomi
 	__isl_take isl_union_map *umap,
 	__isl_take isl_union_pw_qpolynomial_fold *upwf, isl_bool *tight)
 {
-	isl_space *dim;
+	isl_space *space;
 	enum isl_fold type;
 	struct isl_apply_fold_data data;
 
@@ -1656,9 +1663,9 @@ __isl_give isl_union_pw_qpolynomial_fold *isl_union_map_apply_union_pw_qpolynomi
 
 	data.upwf = upwf;
 	data.tight = tight ? isl_bool_true : isl_bool_false;
-	dim = isl_union_pw_qpolynomial_fold_get_space(upwf);
+	space = isl_union_pw_qpolynomial_fold_get_space(upwf);
 	type = isl_union_pw_qpolynomial_fold_get_type(upwf);
-	data.res = isl_union_pw_qpolynomial_fold_zero(dim, type);
+	data.res = isl_union_pw_qpolynomial_fold_zero(space, type);
 	if (isl_union_map_foreach_map(umap, &map_apply, &data) < 0)
 		goto error;
 
@@ -1723,8 +1730,8 @@ __isl_give isl_qpolynomial_fold *isl_qpolynomial_fold_mul_isl_int(
 		return fold;
 	if (fold && isl_int_is_zero(v)) {
 		isl_qpolynomial_fold *zero;
-		isl_space *dim = isl_space_copy(fold->dim);
-		zero = isl_qpolynomial_fold_empty(fold->type, dim);
+		isl_space *space = isl_space_copy(fold->dim);
+		zero = isl_qpolynomial_fold_empty(fold->type, space);
 		isl_qpolynomial_fold_free(fold);
 		return zero;
 	}
