@@ -10,6 +10,11 @@
  * and Ecole Normale Superieure, 45 rue d'Ulm, 75230 Paris, France
  */
 
+#undef TYPE
+#define TYPE	UNION
+static
+#include "has_single_reference_templ.c"
+
 __isl_give UNION *FN(UNION,cow)(__isl_take UNION *u);
 
 isl_ctx *FN(UNION,get_ctx)(__isl_keep UNION *u)
@@ -17,24 +22,33 @@ isl_ctx *FN(UNION,get_ctx)(__isl_keep UNION *u)
 	return u ? u->space->ctx : NULL;
 }
 
-__isl_give isl_space *FN(UNION,get_space)(__isl_keep UNION *u)
+/* Return the space of "u".
+ */
+static __isl_keep isl_space *FN(UNION,peek_space)(__isl_keep UNION *u)
 {
 	if (!u)
 		return NULL;
-	return isl_space_copy(u->space);
+	return u->space;
+}
+
+/* Return a copy of the space of "u".
+ */
+__isl_give isl_space *FN(UNION,get_space)(__isl_keep UNION *u)
+{
+	return isl_space_copy(FN(UNION,peek_space)(u));
 }
 
 /* Return the number of parameters of "u", where "type"
  * is required to be set to isl_dim_param.
  */
-unsigned FN(UNION,dim)(__isl_keep UNION *u, enum isl_dim_type type)
+isl_size FN(UNION,dim)(__isl_keep UNION *u, enum isl_dim_type type)
 {
 	if (!u)
-		return 0;
+		return isl_size_error;
 
 	if (type != isl_dim_param)
 		isl_die(FN(UNION,get_ctx)(u), isl_error_invalid,
-			"can only reference parameters", return 0);
+			"can only reference parameters", return isl_size_error);
 
 	return isl_space_dim(u->space, type);
 }
@@ -51,48 +65,55 @@ int FN(UNION,find_dim_by_name)(__isl_keep UNION *u, enum isl_dim_type type,
 	return isl_space_find_dim_by_name(u->space, type, name);
 }
 
-#ifdef HAS_TYPE
-static __isl_give UNION *FN(UNION,alloc)(__isl_take isl_space *dim,
-	enum isl_fold type, int size)
-#else
-static __isl_give UNION *FN(UNION,alloc)(__isl_take isl_space *dim, int size)
-#endif
+#include "opt_type.h"
+
+static __isl_give UNION *FN(UNION,alloc)(__isl_take isl_space *space
+	OPT_TYPE_PARAM, int size)
 {
 	UNION *u;
 
-	dim = isl_space_params(dim);
-	if (!dim)
+	space = isl_space_params(space);
+	if (!space)
 		return NULL;
 
-	u = isl_calloc_type(dim->ctx, UNION);
+	u = isl_calloc_type(space->ctx, UNION);
 	if (!u)
 		goto error;
 
 	u->ref = 1;
-#ifdef HAS_TYPE
-	u->type = type;
-#endif
-	u->space = dim;
-	if (isl_hash_table_init(dim->ctx, &u->table, size) < 0)
+	OPT_SET_TYPE(u->, type);
+	u->space = space;
+	if (isl_hash_table_init(space->ctx, &u->table, size) < 0)
 		return FN(UNION,free)(u);
 
 	return u;
 error:
-	isl_space_free(dim);
+	isl_space_free(space);
 	return NULL;
 }
 
-#ifdef HAS_TYPE
-__isl_give UNION *FN(UNION,ZERO)(__isl_take isl_space *dim, enum isl_fold type)
+/* Create an empty/zero union without specifying any parameters.
+ */
+__isl_give UNION *FN(FN(UNION,ZERO),ctx)(isl_ctx *ctx OPT_TYPE_PARAM)
 {
-	return FN(UNION,alloc)(dim, type, 16);
+	isl_space *space;
+
+	space = isl_space_unit(ctx);
+	return FN(FN(UNION,ZERO),space)(space OPT_TYPE_ARG(NO_LOC));
 }
-#else
-__isl_give UNION *FN(UNION,ZERO)(__isl_take isl_space *dim)
+
+__isl_give UNION *FN(FN(UNION,ZERO),space)(__isl_take isl_space *space
+	OPT_TYPE_PARAM)
 {
-	return FN(UNION,alloc)(dim, 16);
+	return FN(UNION,alloc)(space OPT_TYPE_ARG(NO_LOC), 16);
+}
+
+/* This is an alternative name for the function above.
+ */
+__isl_give UNION *FN(UNION,ZERO)(__isl_take isl_space *space OPT_TYPE_PARAM)
+{
+	return FN(FN(UNION,ZERO),space)(space OPT_TYPE_ARG(NO_LOC));
 }
-#endif
 
 __isl_give UNION *FN(UNION,copy)(__isl_keep UNION *u)
 {
@@ -103,40 +124,30 @@ __isl_give UNION *FN(UNION,copy)(__isl_keep UNION *u)
 	return u;
 }
 
+/* Do the tuples of "space" correspond to those of the domain of "part"?
+ * That is, is the domain space of "part" equal to "space", ignoring parameters?
+ */
+static isl_bool FN(PART,has_domain_space_tuples)(__isl_keep PART *part,
+	__isl_keep isl_space *space)
+{
+	return isl_space_has_domain_tuples(space, FN(PART,peek_space)(part));
+}
+
 /* Extract the element of "u" living in "space" (ignoring parameters).
  *
  * Return the ZERO element if "u" does not contain any element
  * living in "space".
  */
-__isl_give PART *FN(FN(UNION,extract),PARTS)(__isl_keep UNION *u,
+__isl_give PART *FN(FN(UNION,extract),BASE)(__isl_keep UNION *u,
 	__isl_take isl_space *space)
 {
 	struct isl_hash_table_entry *entry;
-	isl_bool equal_params;
-
-	if (!u || !space)
-		goto error;
-	equal_params = isl_space_has_equal_params(u->space, space);
-	if (equal_params < 0)
-		goto error;
-	if (!equal_params) {
-		space = isl_space_drop_dims(space, isl_dim_param,
-					0, isl_space_dim(space, isl_dim_param));
-		space = isl_space_align_params(space,
-					FN(UNION,get_space)(u));
-		if (!space)
-			goto error;
-	}
 
 	entry = FN(UNION,find_part_entry)(u, space, 0);
 	if (!entry)
 		goto error;
 	if (entry == isl_hash_table_entry_none)
-#ifdef HAS_TYPE
-		return FN(PART,ZERO)(space, u->type);
-#else
-		return FN(PART,ZERO)(space);
-#endif
+		return FN(PART,ZERO)(space OPT_TYPE_ARG(u->));
 	isl_space_free(space);
 	return FN(PART,copy)(entry->data);
 error:
@@ -210,14 +221,13 @@ error:
 /* Add "part" to "u", where "u" is assumed not to already have
  * a part that is defined on the same space as "part".
  */
-__isl_give UNION *FN(FN(UNION,add),PARTS)(__isl_take UNION *u,
+__isl_give UNION *FN(FN(UNION,add),BASE)(__isl_take UNION *u,
 	__isl_take PART *part)
 {
 	return FN(UNION,add_part_generic)(u, part, 1);
 }
 
-#ifdef HAS_TYPE
-/* Allocate a UNION with the same type and the same size as "u" and
+/* Allocate a UNION with the same type (if any) and the same size as "u" and
  * with space "space".
  */
 static __isl_give UNION *FN(UNION,alloc_same_size_on_space)(__isl_keep UNION *u,
@@ -225,25 +235,11 @@ static __isl_give UNION *FN(UNION,alloc_same_size_on_space)(__isl_keep UNION *u,
 {
 	if (!u)
 		goto error;
-	return FN(UNION,alloc)(space, u->type, u->table.n);
+	return FN(UNION,alloc)(space OPT_TYPE_ARG(u->), u->table.n);
 error:
 	isl_space_free(space);
 	return NULL;
 }
-#else
-/* Allocate a UNION with the same size as "u" and with space "space".
- */
-static __isl_give UNION *FN(UNION,alloc_same_size_on_space)(__isl_keep UNION *u,
-	__isl_take isl_space *space)
-{
-	if (!u)
-		goto error;
-	return FN(UNION,alloc)(space, u->table.n);
-error:
-	isl_space_free(space);
-	return NULL;
-}
-#endif
 
 /* Allocate a UNION with the same space, the same type (if any) and
  * the same size as "u".
@@ -253,68 +249,108 @@ static __isl_give UNION *FN(UNION,alloc_same_size)(__isl_keep UNION *u)
 	return FN(UNION,alloc_same_size_on_space)(u, FN(UNION,get_space)(u));
 }
 
+/* Data structure that specifies how isl_union_*_transform
+ * should modify the base expressions in the union expression.
+ *
+ * If "inplace" is set, then the base expression in the input union
+ * are modified in place.  This means that "fn" should not
+ * change the meaning of the union or that the union only
+ * has a single reference.
+ * If "space" is not NULL, then a new union is created in this space.
+ * If "filter" is not NULL, then only the base expressions that satisfy "filter"
+ * are taken into account.
+ * "filter_user" is passed as the second argument to "filter".
+ * If "fn" it not NULL, then it is applied to each entry in the input.
+ * "fn_user" is passed as the second argument to "fn".
+ */
+S(UNION,transform_control) {
+	int inplace;
+	isl_space *space;
+	isl_bool (*filter)(__isl_keep PART *part, void *user);
+	void *filter_user;
+	__isl_give PART *(*fn)(__isl_take PART *part, void *user);
+	void *fn_user;
+};
+
 /* Internal data structure for isl_union_*_transform_space.
- * "fn' is applied to each entry in the input.
- * "res" collects the results.
+ * "control" specifies how the base expressions should be modified.
+ * "res" collects the results (if control->inplace is not set).
  */
 S(UNION,transform_data)
 {
-	__isl_give PART *(*fn)(__isl_take PART *part, void *user);
-	void *user;
-
+	S(UNION,transform_control) *control;
 	UNION *res;
 };
 
-/* Apply data->fn to "part" and add the result to data->res.
+/* Apply control->fn to "part" and add the result to data->res or
+ * place it back into the input union if control->inplace is set.
  */
-static isl_stat FN(UNION,transform_entry)(__isl_take PART *part, void *user)
+static isl_stat FN(UNION,transform_entry)(void **entry, void *user)
 {
 	S(UNION,transform_data) *data = (S(UNION,transform_data) *)user;
+	S(UNION,transform_control) *control = data->control;
+	PART *part = *entry;
 
-	part = data->fn(part, data->user);
-	data->res = FN(FN(UNION,add),PARTS)(data->res, part);
-	if (!data->res)
+	if (control->filter) {
+		isl_bool handle;
+
+		handle = control->filter(part, control->filter_user);
+		if (handle < 0)
+			return isl_stat_error;
+		if (!handle)
+			return isl_stat_ok;
+	}
+
+	if (!control->inplace)
+		part = FN(PART,copy)(part);
+	if (control->fn)
+		part = control->fn(part, control->fn_user);
+	if (control->inplace)
+		*entry = part;
+	else
+		data->res = FN(FN(UNION,add),BASE)(data->res, part);
+	if (!part || !data->res)
 		return isl_stat_error;
 
 	return isl_stat_ok;
 }
 
-/* Return a UNION living in "space" that is obtained by applying "fn"
- * to each of the entries in "u".
+/* Return a UNION that is obtained by modifying "u" according to "control".
  */
-static __isl_give UNION *FN(UNION,transform_space)(__isl_take UNION *u,
-	isl_space *space,
-	__isl_give PART *(*fn)(__isl_take PART *part, void *user), void *user)
+static __isl_give UNION *FN(UNION,transform)(__isl_take UNION *u,
+	S(UNION,transform_control) *control)
 {
-	S(UNION,transform_data) data = { fn, user };
+	S(UNION,transform_data) data = { control };
+	isl_space *space;
 
-	data.res = FN(UNION,alloc_same_size_on_space)(u, space);
-	if (FN(FN(UNION,foreach),PARTS)(u,
-					&FN(UNION,transform_entry), &data) < 0)
+	if (control->inplace) {
+		data.res = u;
+	} else {
+		if (control->space)
+			space = isl_space_copy(control->space);
+		else
+			space = FN(UNION,get_space)(u);
+		data.res = FN(UNION,alloc_same_size_on_space)(u, space);
+	}
+	if (FN(UNION,foreach_inplace)(u, &FN(UNION,transform_entry), &data) < 0)
 		data.res = FN(UNION,free)(data.res);
-	FN(UNION,free)(u);
+	if (!control->inplace)
+		FN(UNION,free)(u);
 	return data.res;
 }
 
-/* Return a UNION that lives in the same space as "u" and that is obtained
- * by applying "fn" to each of the entries in "u".
+/* Return a UNION living in "space" that is otherwise obtained by modifying "u"
+ * according to "control".
  */
-static __isl_give UNION *FN(UNION,transform)(__isl_take UNION *u,
-	__isl_give PART *(*fn)(__isl_take PART *part, void *user), void *user)
+static __isl_give UNION *FN(UNION,transform_space)(__isl_take UNION *u,
+	__isl_take isl_space *space, S(UNION,transform_control) *control)
 {
-	return FN(UNION,transform_space)(u, FN(UNION,get_space)(u), fn, user);
-}
-
-/* Apply data->fn to *part and store the result back into *part.
- */
-static isl_stat FN(UNION,transform_inplace_entry)(void **part, void *user)
-{
-	S(UNION,transform_data) *data = (S(UNION,transform_data) *) user;
-
-	*part = data->fn(*part, data->user);
-	if (!*part)
-		return isl_stat_error;
-	return isl_stat_ok;
+	if (!space)
+		return FN(UNION,free)(u);
+	control->space = space;
+	u = FN(UNION,transform)(u, control);
+	isl_space_free(space);
+	return u;
 }
 
 /* Update "u" by applying "fn" to each entry.
@@ -327,19 +363,15 @@ static isl_stat FN(UNION,transform_inplace_entry)(void **part, void *user)
 static __isl_give UNION *FN(UNION,transform_inplace)(__isl_take UNION *u,
 	__isl_give PART *(*fn)(__isl_take PART *part, void *user), void *user)
 {
+	S(UNION,transform_control) control = { .fn = fn, .fn_user = user };
 	isl_bool single_ref;
 
 	single_ref = FN(UNION,has_single_reference)(u);
 	if (single_ref < 0)
 		return FN(UNION,free)(u);
-	if (single_ref) {
-		S(UNION,transform_data) data = { fn, user };
-		if (FN(UNION,foreach_inplace)(u,
-				&FN(UNION,transform_inplace_entry), &data) < 0)
-			return FN(UNION,free)(u);
-		return u;
-	}
-	return FN(UNION,transform)(u, fn, user);
+	if (single_ref)
+		control.inplace = 1;
+	return FN(UNION,transform)(u, &control);
 }
 
 /* An isl_union_*_transform callback for use in isl_union_*_dup
@@ -352,8 +384,10 @@ static __isl_give PART *FN(UNION,copy_part)(__isl_take PART *part, void *user)
 
 __isl_give UNION *FN(UNION,dup)(__isl_keep UNION *u)
 {
+	S(UNION,transform_control) control = { .fn = &FN(UNION,copy_part) };
+
 	u = FN(UNION,copy)(u);
-	return FN(UNION,transform)(u, &FN(UNION,copy_part), NULL);
+	return FN(UNION,transform)(u, &control);
 }
 
 __isl_give UNION *FN(UNION,cow)(__isl_take UNION *u)
@@ -397,13 +431,17 @@ static __isl_give PART *FN(UNION,align_entry)(__isl_take PART *part, void *user)
 static __isl_give UNION *FN(UNION,realign_domain)(__isl_take UNION *u,
 	__isl_take isl_reordering *r)
 {
+	S(UNION,transform_control) control = {
+		.fn = &FN(UNION,align_entry),
+		.fn_user = r,
+	};
 	isl_space *space;
 
 	if (!u || !r)
 		goto error;
 
-	space = isl_space_copy(r->dim);
-	u = FN(UNION,transform_space)(u, space, &FN(UNION,align_entry), r);
+	space = isl_reordering_get_space(r);
+	u = FN(UNION,transform_space)(u, space, &control);
 	isl_reordering_free(r);
 	return u;
 error:
@@ -431,7 +469,6 @@ __isl_give UNION *FN(UNION,align_params)(__isl_take UNION *u,
 		return u;
 	}
 
-	model = isl_space_params(model);
 	r = isl_parameter_alignment_reordering(u->space, model);
 	isl_space_free(model);
 
@@ -475,7 +512,7 @@ static __isl_give UNION *FN(UNION,union_add_)(__isl_take UNION *u1,
 	if (!u1 || !u2)
 		goto error;
 
-	if (FN(FN(UNION,foreach),PARTS)(u2, &FN(UNION,union_add_part), &u1) < 0)
+	if (FN(FN(UNION,foreach),BASE)(u2, &FN(UNION,union_add_part), &u1) < 0)
 		goto error;
 
 	FN(UNION,free)(u2);
@@ -487,23 +524,21 @@ error:
 	return NULL;
 }
 
-__isl_give UNION *FN(FN(UNION,from),PARTS)(__isl_take PART *part)
+__isl_give UNION *FN(FN(UNION,from),BASE)(__isl_take PART *part)
 {
-	isl_space *dim;
+	isl_space *space;
 	UNION *u;
 
 	if (!part)
 		return NULL;
 
-	dim = FN(PART,get_space)(part);
-	dim = isl_space_drop_dims(dim, isl_dim_in, 0, isl_space_dim(dim, isl_dim_in));
-	dim = isl_space_drop_dims(dim, isl_dim_out, 0, isl_space_dim(dim, isl_dim_out));
-#ifdef HAS_TYPE
-	u = FN(UNION,ZERO)(dim, part->type);
-#else
-	u = FN(UNION,ZERO)(dim);
-#endif
-	u = FN(FN(UNION,add),PARTS)(u, part);
+	space = FN(PART,get_space)(part);
+	space = isl_space_drop_dims(space, isl_dim_in, 0,
+					isl_space_dim(space, isl_dim_in));
+	space = isl_space_drop_dims(space, isl_dim_out, 0,
+					isl_space_dim(space, isl_dim_out));
+	u = FN(UNION,ZERO)(space OPT_TYPE_ARG(part->));
+	u = FN(FN(UNION,add),BASE)(u, part);
 
 	return u;
 }
@@ -544,7 +579,7 @@ static isl_stat FN(UNION,match_bin_entry)(__isl_take PART *part, void *user)
 
 	part = data->fn(part, FN(PART, copy)(entry2->data));
 
-	data->res = FN(FN(UNION,add),PARTS)(data->res, part);
+	data->res = FN(FN(UNION,add),BASE)(data->res, part);
 	if (!data->res)
 		return isl_stat_error;
 
@@ -578,7 +613,7 @@ static __isl_give UNION *FN(UNION,match_bin_op)(__isl_take UNION *u1,
 
 	data.u2 = u2;
 	data.res = FN(UNION,alloc_same_size)(u1);
-	if (FN(FN(UNION,foreach),PARTS)(u1,
+	if (FN(FN(UNION,foreach),BASE)(u1,
 				    &FN(UNION,match_bin_entry), &data) < 0)
 		goto error;
 
@@ -636,6 +671,10 @@ static __isl_give UNION *FN(UNION,any_set_op)(__isl_take UNION *u,
 	__isl_give PW *(*fn)(__isl_take PW*, __isl_take isl_set*))
 {
 	S(UNION,any_set_data) data = { NULL, fn };
+	S(UNION,transform_control) control = {
+		.fn = &FN(UNION,any_set_entry),
+		.fn_user = &data,
+	};
 
 	u = FN(UNION,align_params)(u, isl_set_get_space(set));
 	set = isl_set_align_params(set, FN(UNION,get_space)(u));
@@ -644,7 +683,7 @@ static __isl_give UNION *FN(UNION,any_set_op)(__isl_take UNION *u,
 		goto error;
 
 	data.set = set;
-	u = FN(UNION,transform)(u, &FN(UNION,any_set_entry), &data);
+	u = FN(UNION,transform)(u, &control);
 	isl_set_free(set);
 	return u;
 error:
@@ -670,69 +709,82 @@ __isl_give UNION *FN(UNION,gist_params)(__isl_take UNION *u,
 	return FN(UNION,any_set_op)(u, set, &FN(PW,gist_params));
 }
 
-S(UNION,match_domain_data) {
-	isl_union_set *uset;
-	UNION *res;
+/* Data structure that specifies how isl_union_*_match_domain_op
+ * should combine its arguments.
+ *
+ * If "filter" is not NULL, then only parts that pass the given
+ * filter are considered for matching.
+ * "fn" is applied to each part in the union and each corresponding
+ * set in the union set, i.e., such that the set lives in the same space
+ * as the domain of the part.
+ * If "match_space" is not NULL, then the set extracted from the union set
+ * does not live in the same space as the domain of the part,
+ * but rather in the space that results from calling "match_space"
+ * on this domain space.
+ */
+S(UNION,match_domain_control) {
+	isl_bool (*filter)(__isl_keep PART *part);
+	__isl_give isl_space *(*match_space)(__isl_take isl_space *space);
 	__isl_give PW *(*fn)(__isl_take PW*, __isl_take isl_set*);
 };
 
-static int FN(UNION,set_has_dim)(const void *entry, const void *val)
-{
-	isl_set *set = (isl_set *)entry;
-	isl_space *dim = (isl_space *)val;
-
-	return isl_space_is_equal(set->dim, dim);
-}
+S(UNION,match_domain_data) {
+	isl_union_set *uset;
+	UNION *res;
+	S(UNION,match_domain_control) *control;
+};
 
 /* Find the set in data->uset that lives in the same space as the domain
- * of "part", apply data->fn to *entry and this set (if any), and add
- * the result to data->res.
+ * of "part" (ignoring parameters), apply data->fn to *entry and this set
+ * (if any), and add the result to data->res.
  */
 static isl_stat FN(UNION,match_domain_entry)(__isl_take PART *part, void *user)
 {
 	S(UNION,match_domain_data) *data = user;
-	uint32_t hash;
 	struct isl_hash_table_entry *entry2;
 	isl_space *space;
 
-	space = FN(PART,get_domain_space)(part);
-	hash = isl_space_get_hash(space);
-	entry2 = isl_hash_table_find(data->uset->dim->ctx, &data->uset->table,
-				     hash, &FN(UNION,set_has_dim), space, 0);
-	isl_space_free(space);
-	if (!entry2) {
-		FN(PART,free)(part);
-		return isl_stat_ok;
+	if (data->control->filter) {
+		isl_bool pass = data->control->filter(part);
+		if (pass < 0 || !pass) {
+			FN(PART,free)(part);
+			return pass < 0 ? isl_stat_error : isl_stat_ok;
+		}
 	}
 
-	part = data->fn(part, isl_set_copy(entry2->data));
+	space = FN(PART,get_domain_space)(part);
+	if (data->control->match_space)
+		space = data->control->match_space(space);
+	entry2 = isl_union_set_find_entry(data->uset, space, 0);
+	isl_space_free(space);
+	if (!entry2 || entry2 == isl_hash_table_entry_none) {
+		FN(PART,free)(part);
+		return isl_stat_non_null(entry2);
+	}
 
-	data->res = FN(FN(UNION,add),PARTS)(data->res, part);
+	part = data->control->fn(part, isl_set_copy(entry2->data));
+
+	data->res = FN(FN(UNION,add),BASE)(data->res, part);
 	if (!data->res)
 		return isl_stat_error;
 
 	return isl_stat_ok;
 }
 
-/* Apply fn to each pair of PW in u and set in uset such that
- * the set lives in the same space as the domain of PW
+/* Combine "u" and "uset" according to "control"
  * and collect the results.
  */
 static __isl_give UNION *FN(UNION,match_domain_op)(__isl_take UNION *u,
-	__isl_take isl_union_set *uset,
-	__isl_give PW *(*fn)(__isl_take PW*, __isl_take isl_set*))
+	__isl_take isl_union_set *uset, S(UNION,match_domain_control) *control)
 {
-	S(UNION,match_domain_data) data = { NULL, NULL, fn };
-
-	u = FN(UNION,align_params)(u, isl_union_set_get_space(uset));
-	uset = isl_union_set_align_params(uset, FN(UNION,get_space)(u));
+	S(UNION,match_domain_data) data = { NULL, NULL, control };
 
 	if (!u || !uset)
 		goto error;
 
 	data.uset = uset;
 	data.res = FN(UNION,alloc_same_size)(u);
-	if (FN(FN(UNION,foreach),PARTS)(u,
+	if (FN(FN(UNION,foreach),BASE)(u,
 				   &FN(UNION,match_domain_entry), &data) < 0)
 		goto error;
 
@@ -750,13 +802,92 @@ error:
  * If "uset" is a parameters domain, then intersect the parameter
  * domain of "u" with this set.
  */
-__isl_give UNION *FN(UNION,intersect_domain)(__isl_take UNION *u,
+__isl_give UNION *FN(UNION,intersect_domain_union_set)(__isl_take UNION *u,
 	__isl_take isl_union_set *uset)
 {
+	S(UNION,match_domain_control) control = {
+		.fn = &FN(PW,intersect_domain),
+	};
+
 	if (isl_union_set_is_params(uset))
 		return FN(UNION,intersect_params)(u,
 						isl_set_from_union_set(uset));
-	return FN(UNION,match_domain_op)(u, uset, &FN(PW,intersect_domain));
+	return FN(UNION,match_domain_op)(u, uset, &control);
+}
+
+/* This is an alternative name for the function above.
+ */
+__isl_give UNION *FN(UNION,intersect_domain)(__isl_take UNION *u,
+	__isl_take isl_union_set *uset)
+{
+	return FN(UNION,intersect_domain_union_set)(u, uset);
+}
+
+/* Return true if this part should be kept.
+ *
+ * In particular, it should be kept if its domain space
+ * corresponds to "space".
+ */
+static isl_bool FN(UNION,select_entry)(__isl_keep PART *part, void *user)
+{
+	isl_space *space = user;
+
+	return FN(PW,has_domain_space_tuples)(part, space);
+}
+
+/* Remove any not element in "space" from the domain of "u".
+ *
+ * In particular, select any part of the function defined
+ * on this domain space.
+ */
+__isl_give UNION *FN(UNION,intersect_domain_space)(__isl_take UNION *u,
+	__isl_take isl_space *space)
+{
+	S(UNION,transform_control) control = {
+		.filter = &FN(UNION,select_entry),
+		.filter_user = space,
+	};
+
+	u = FN(UNION,transform)(u, &control);
+	isl_space_free(space);
+	return u;
+}
+
+/* Is the domain of "pw" a wrapped relation?
+ */
+static isl_bool FN(PW,domain_is_wrapping)(__isl_keep PW *pw)
+{
+	return isl_space_domain_is_wrapping(FN(PW,peek_space)(pw));
+}
+
+/* Intersect the domain of the wrapped relation inside the domain of "u"
+ * with "uset".
+ */
+__isl_give UNION *FN(UNION,intersect_domain_wrapped_domain)(__isl_take UNION *u,
+	__isl_take isl_union_set *uset)
+{
+	S(UNION,match_domain_control) control = {
+		.filter = &FN(PART,domain_is_wrapping),
+		.match_space = &isl_space_factor_domain,
+		.fn = &FN(PW,intersect_domain_wrapped_domain),
+	};
+
+	return FN(UNION,match_domain_op)(u, uset, &control);
+}
+
+/* Intersect the range of the wrapped relation inside the domain of "u"
+ * with "uset".
+ */
+__isl_give UNION *FN(UNION,intersect_domain_wrapped_range)(__isl_take UNION *u,
+	__isl_take isl_union_set *uset)
+{
+	S(UNION,match_domain_control) control = {
+		.filter = &FN(PART,domain_is_wrapping),
+		.match_space = &isl_space_factor_range,
+		.fn = &FN(PW,intersect_domain_wrapped_range),
+	};
+
+	return FN(UNION,match_domain_op)(u, uset, &control);
 }
 
 /* Take the set (which may be empty) in data->uset that lives
@@ -775,22 +906,69 @@ static __isl_give PART *FN(UNION,subtract_domain_entry)(__isl_take PART *part,
 	return FN(PART,subtract_domain)(part, set);
 }
 
-/* Subtract "uset' from the domain of "u".
+/* Subtract "uset" from the domain of "u".
+ */
+__isl_give UNION *FN(UNION,subtract_domain_union_set)(__isl_take UNION *u,
+	__isl_take isl_union_set *uset)
+{
+	S(UNION,transform_control) control = {
+		.fn = &FN(UNION,subtract_domain_entry),
+		.fn_user = uset,
+	};
+
+	u = FN(UNION,transform)(u, &control);
+	isl_union_set_free(uset);
+	return u;
+}
+
+/* This is an alternative name for the function above.
  */
 __isl_give UNION *FN(UNION,subtract_domain)(__isl_take UNION *u,
 	__isl_take isl_union_set *uset)
 {
-	u = FN(UNION,transform)(u, &FN(UNION,subtract_domain_entry), uset);
-	isl_union_set_free(uset);
+	return FN(UNION,subtract_domain_union_set)(u, uset);
+}
+
+/* Return true if this part should be kept.
+ *
+ * In particular, it should be kept if its domain space
+ * does not correspond to "space".
+ */
+static isl_bool FN(UNION,filter_out_entry)(__isl_keep PART *part, void *user)
+{
+	isl_space *space = user;
+
+	return isl_bool_not(FN(PW,has_domain_space_tuples)(part, space));
+}
+
+/* Remove any element in "space" from the domain of "u".
+ *
+ * In particular, filter out any part of the function defined
+ * on this domain space.
+ */
+__isl_give UNION *FN(UNION,subtract_domain_space)(__isl_take UNION *u,
+	__isl_take isl_space *space)
+{
+	S(UNION,transform_control) control = {
+		.filter = &FN(UNION,filter_out_entry),
+		.filter_user = space,
+	};
+
+	u = FN(UNION,transform)(u, &control);
+	isl_space_free(space);
 	return u;
 }
 
 __isl_give UNION *FN(UNION,gist)(__isl_take UNION *u,
 	__isl_take isl_union_set *uset)
 {
+	S(UNION,match_domain_control) control = {
+		.fn = &FN(PW,gist),
+	};
+
 	if (isl_union_set_is_params(uset))
 		return FN(UNION,gist_params)(u, isl_set_from_union_set(uset));
-	return FN(UNION,match_domain_op)(u, uset, &FN(PW,gist));
+	return FN(UNION,match_domain_op)(u, uset, &control);
 }
 
 /* Coalesce an entry in a UNION.  Coalescing is performed in-place.
@@ -837,7 +1015,7 @@ __isl_give isl_union_set *FN(UNION,domain)(__isl_take UNION *u)
 	isl_union_set *uset;
 
 	uset = isl_union_set_empty(FN(UNION,get_space)(u));
-	if (FN(FN(UNION,foreach),PARTS)(u, &FN(UNION,domain_entry), &uset) < 0)
+	if (FN(FN(UNION,foreach),BASE)(u, &FN(UNION,domain_entry), &uset) < 0)
 		goto error;
 
 	FN(UNION,free)(u);
@@ -895,11 +1073,7 @@ __isl_give UNION *FN(UNION,scale_val)(__isl_take UNION *u,
 	if (DEFAULT_IS_ZERO && u && isl_val_is_zero(v)) {
 		UNION *zero;
 		isl_space *space = FN(UNION,get_space)(u);
-#ifdef HAS_TYPE
-		zero = FN(UNION,ZERO)(space, u->type);
-#else
-		zero = FN(UNION,ZERO)(space);
-#endif
+		zero = FN(UNION,ZERO)(space OPT_TYPE_ARG(u->));
 		FN(UNION,free)(u);
 		isl_val_free(v);
 		return zero;
@@ -962,38 +1136,75 @@ error:
 	return NULL;
 }
 
-S(UNION,plain_is_equal_data)
-{
-	UNION *u2;
-	isl_bool is_equal;
+/* Internal data structure for isl_union_*_every_*.
+ *
+ * "test" is the user-specified callback function.
+ * "user" is the user-specified callback function argument.
+ *
+ * "res" is the final result, initialized to isl_bool_true.
+ */
+S(UNION,every_data) {
+	isl_bool (*test)(__isl_keep PW *pw, void *user);
+	void *user;
+
+	isl_bool res;
 };
 
-static isl_stat FN(UNION,plain_is_equal_entry)(void **entry, void *user)
+/* Call data->test on the piecewise expression at *entry,
+ * updating the result in data->res.
+ * Abort if this result is no longer isl_bool_true.
+ */
+static isl_stat FN(UNION,every_entry)(void **entry, void *user)
 {
-	S(UNION,plain_is_equal_data) *data = user;
-	struct isl_hash_table_entry *entry2;
+	S(UNION,every_data) *data = user;
 	PW *pw = *entry;
 
-	entry2 = FN(UNION,find_part_entry)(data->u2, pw->dim, 0);
-	if (!entry2 || entry2 == isl_hash_table_entry_none) {
-		if (!entry2)
-			data->is_equal = isl_bool_error;
-		else
-			data->is_equal = isl_bool_false;
-		return isl_stat_error;
-	}
-
-	data->is_equal = FN(PW,plain_is_equal)(pw, entry2->data);
-	if (data->is_equal < 0 || !data->is_equal)
+	data->res = data->test(pw, data->user);
+	if (data->res < 0 || !data->res)
 		return isl_stat_error;
 
 	return isl_stat_ok;
 }
 
+/* Does "test" succeed on every piecewise expression in "u"?
+ */
+isl_bool FN(FN(UNION,every),BASE)(__isl_keep UNION *u,
+	isl_bool (*test)(__isl_keep PW *pw, void *user), void *user)
+{
+	S(UNION,every_data) data = { test, user };
+
+	data.res = isl_bool_true;
+	if (FN(UNION,foreach_inplace)(u, &FN(UNION,every_entry), &data) < 0 &&
+	    data.res == isl_bool_true)
+		return isl_bool_error;
+
+	return data.res;
+}
+
+S(UNION,plain_is_equal_data)
+{
+	UNION *u2;
+};
+
+static isl_bool FN(UNION,plain_is_equal_el)(__isl_keep PW *pw, void *user)
+{
+	S(UNION,plain_is_equal_data) *data = user;
+	struct isl_hash_table_entry *entry;
+
+	entry = FN(UNION,find_part_entry)(data->u2, pw->dim, 0);
+	if (!entry)
+		return isl_bool_error;
+	if (entry == isl_hash_table_entry_none)
+		return isl_bool_false;
+
+	return FN(PW,plain_is_equal)(pw, entry->data);
+}
+
 isl_bool FN(UNION,plain_is_equal)(__isl_keep UNION *u1, __isl_keep UNION *u2)
 {
-	S(UNION,plain_is_equal_data) data = { NULL, isl_bool_true };
-	int n1, n2;
+	S(UNION,plain_is_equal_data) data;
+	isl_size n1, n2;
+	isl_bool is_equal;
 
 	if (!u1 || !u2)
 		return isl_bool_error;
@@ -1001,8 +1212,8 @@ isl_bool FN(UNION,plain_is_equal)(__isl_keep UNION *u1, __isl_keep UNION *u2)
 		return isl_bool_true;
 	if (u1->table.n != u2->table.n)
 		return isl_bool_false;
-	n1 = FN(FN(UNION,n),PARTS)(u1);
-	n2 = FN(FN(UNION,n),PARTS)(u2);
+	n1 = FN(FN(UNION,n),BASE)(u1);
+	n2 = FN(FN(UNION,n),BASE)(u2);
 	if (n1 < 0 || n2 < 0)
 		return isl_bool_error;
 	if (n1 != n2)
@@ -1016,52 +1227,36 @@ isl_bool FN(UNION,plain_is_equal)(__isl_keep UNION *u1, __isl_keep UNION *u2)
 		goto error;
 
 	data.u2 = u2;
-	if (FN(UNION,foreach_inplace)(u1,
-			       &FN(UNION,plain_is_equal_entry), &data) < 0 &&
-	    data.is_equal)
-		goto error;
+	is_equal = FN(FN(UNION,every),BASE)(u1,
+					  &FN(UNION,plain_is_equal_el), &data);
 
 	FN(UNION,free)(u1);
 	FN(UNION,free)(u2);
 
-	return data.is_equal;
+	return is_equal;
 error:
 	FN(UNION,free)(u1);
 	FN(UNION,free)(u2);
 	return isl_bool_error;
 }
 
-/* Check whether the element that "entry" points to involves any NaNs and
- * store the result in *nan.
- * Abort as soon as one such element has been found.
+/* An isl_union_*_every_* callback that checks whether "pw"
+ * does not involve any NaNs.
  */
-static isl_stat FN(UNION,involves_nan_entry)(void **entry, void *user)
+static isl_bool FN(UNION,no_nan_el)(__isl_keep PW *pw, void *user)
 {
-	isl_bool *nan = user;
-	PW *pw = *entry;
-
-	*nan = FN(PW,involves_nan)(pw);
-	if (*nan < 0 || !nan)
-		return isl_stat_error;
-
-	return isl_stat_ok;
+	return isl_bool_not(FN(PW,involves_nan)(pw));
 }
 
 /* Does "u" involve any NaNs?
  */
 isl_bool FN(UNION,involves_nan)(__isl_keep UNION *u)
 {
-	isl_bool nan = isl_bool_false;
+	isl_bool no_nan;
 
-	if (!u)
-		return isl_bool_error;
+	no_nan = FN(FN(UNION,every),BASE)(u, &FN(UNION,no_nan_el), NULL);
 
-	if (FN(UNION,foreach_inplace)(u,
-				    &FN(UNION,involves_nan_entry), &nan) < 0 &&
-	    !nan)
-		return isl_bool_error;
-
-	return nan;
+	return isl_bool_not(no_nan);
 }
 
 /* Internal data structure for isl_union_*_drop_dims.
@@ -1091,6 +1286,10 @@ __isl_give UNION *FN(UNION,drop_dims)( __isl_take UNION *u,
 {
 	isl_space *space;
 	S(UNION,drop_dims_data) data = { type, first, n };
+	S(UNION,transform_control) control = {
+		.fn = &FN(UNION,drop_dims_entry),
+		.fn_user = &data,
+	};
 
 	if (!u)
 		return NULL;
@@ -1102,8 +1301,7 @@ __isl_give UNION *FN(UNION,drop_dims)( __isl_take UNION *u,
 
 	space = FN(UNION,get_space)(u);
 	space = isl_space_drop_dims(space, type, first, n);
-	return FN(UNION,transform_space)(u, space, &FN(UNION,drop_dims_entry),
-					&data);
+	return FN(UNION,transform_space)(u, space, &control);
 }
 
 /* Internal data structure for isl_union_*_set_dim_name.
@@ -1133,6 +1331,10 @@ __isl_give UNION *FN(UNION,set_dim_name)(__isl_take UNION *u,
 	enum isl_dim_type type, unsigned pos, const char *s)
 {
 	S(UNION,set_dim_name_data) data = { pos, s };
+	S(UNION,transform_control) control = {
+		.fn = &FN(UNION,set_dim_name_entry),
+		.fn_user = &data,
+	};
 	isl_space *space;
 
 	if (!u)
@@ -1145,8 +1347,7 @@ __isl_give UNION *FN(UNION,set_dim_name)(__isl_take UNION *u,
 
 	space = FN(UNION,get_space)(u);
 	space = isl_space_set_dim_name(space, type, pos, s);
-	return FN(UNION,transform_space)(u, space,
-					&FN(UNION,set_dim_name_entry), &data);
+	return FN(UNION,transform_space)(u, space, &control);
 }
 
 /* Reset the user pointer on all identifiers of parameters and tuples
@@ -1163,10 +1364,48 @@ static __isl_give PART *FN(UNION,reset_user_entry)(__isl_take PART *part,
  */
 __isl_give UNION *FN(UNION,reset_user)(__isl_take UNION *u)
 {
+	S(UNION,transform_control) control = {
+		.fn = &FN(UNION,reset_user_entry),
+	};
 	isl_space *space;
 
 	space = FN(UNION,get_space)(u);
 	space = isl_space_reset_user(space);
-	return FN(UNION,transform_space)(u, space, &FN(UNION,reset_user_entry),
-					NULL);
+	return FN(UNION,transform_space)(u, space, &control);
+}
+
+/* Add the base expression held by "entry" to "list".
+ */
+static isl_stat FN(UNION,add_to_list)(void **entry, void *user)
+{
+	PW *pw = *entry;
+	LIST(PART) **list = user;
+
+	*list = FN(LIST(PART),add)(*list, FN(PART,copy)(pw));
+	if (!*list)
+		return isl_stat_error;
+
+	return isl_stat_ok;
+}
+
+/* Return a list containing all the base expressions in "u".
+ *
+ * First construct a list of the appropriate size and
+ * then add all the elements.
+ */
+__isl_give LIST(PART) *FN(FN(UNION,get),LIST(BASE))(__isl_keep UNION *u)
+{
+	isl_size n;
+	LIST(PART) *list;
+
+	if (!u)
+		return NULL;
+	n = FN(FN(UNION,n),BASE)(u);
+	if (n < 0)
+		return NULL;
+	list = FN(LIST(PART),alloc)(FN(UNION,get_ctx(u)), n);
+	if (FN(UNION,foreach_inplace)(u, &FN(UNION,add_to_list), &list) < 0)
+		return FN(LIST(PART),free)(list);
+
+	return list;
 }
